#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a painting on satin

# (c) Dinasset 2022
#
#   History:
#
#   v1.0: 2022-03: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
from gimpfu import *

# define here some constant value, when required
#FixWH=3000

############################################################################
# Service routines
#---------------------------------------------------------------------------
  
#---------------------------------------- 
# for messages use: erro_msg or info_msg
#---------------------------------------- 
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

#----------------------------------------------- 
# from Original drawable create the Output image
#----------------------------------------------- 
def createWrkimage(inImage, inLayer, WrkWH):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    OrigWH = inLayer.width+inLayer.height    
    
   # define/calculate the scale factor for the (new) output image  
    scale = (WrkWH*1.0)/(OrigWH*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size  
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
    # adjust mean values
    adjustmeanv (newImage, newBG)
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
   
    newBG.opacity = 100.0
    
    return(newImage)
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# adjust values and saturation
#---------------------------------------------------------------------------
def adjustmeanSV (inImage, inLayer):
    # decompose HSV
    (HSVI,HL,SL,VL) = pdb.plug_in_decompose (inImage, inLayer, "HSV", True)
    #pdb.gimp_display_new (HSVI)
    HL=HSVI.layers[0]; SL=HSVI.layers[1]; VL=HSVI.layers[2];
    adjustmeanv (HSVI, SL)
    adjustmeanv (HSVI, VL)
    pdb.plug_in_recompose (HSVI, HL)
    return(inLayer)

#---------------------------------------------------------------------------
# adjust values
#---------------------------------------------------------------------------
def adjustmeanv(inImage, inLayer):
    # take care of the existing mean in determining the theoretical mean
    meanadj=50
    # check mean value of the layer
    (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                  HISTOGRAM_VALUE, 0.0, 1.0)
    thmean = (127.5*meanadj/100+meanl*(100.0-meanadj)/100)   
    #info_msg ("thmean="+str(thmean))
    # range for the loops
    minmean = thmean*1.05  
    maxmean = thmean*1.15  
    #info_msg ("target min mean value="+str(minmean))
    #info_msg ("target max mean value="+str(maxmean))
    
    if meanl<minmean:
        while meanl<minmean:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 1.1,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
    elif meanl>maxmean:
        while meanl>maxmean:
            #info_msg ("mean source value="+str(meanl))
            pdb.gimp_drawable_levels (inLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False, 0.9,
                                      0.0, 1.0, False)
            (meanl,std,med,pix,cnt,per) = pdb.gimp_drawable_histogram (inLayer,
                                          HISTOGRAM_VALUE, 0.0, 1.0)
    
    
    return(inLayer)

############################################################################
# Main
#---------------------------------------------------------------------------

def MosaicTextureColoree (inImage, inLayer,
                   UserOutWH, Colour,
                   TextureLook,  
                   doFlatten):
    
    # here the logic of the filter
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    OrigWH = inLayer.width+inLayer.height
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
      
    # Set Colour (default = Sanguigna) and check
    (RED,GRE,BLU,A) = Colour  # input parameter
    if ((RED+GRE+BLU)/3) > 159:     # 85
        info_msg ("please re-enter nuance, must be darker ")
        return
    
    # create new image to work on
    WorkWH = 1000+500*int(TextureLook)
    newImage = createWrkimage (inImage, inLayer, WorkWH)
    width = newImage.width
    height = newImage.height
    Preprocessed = newImage.layers[0]
    Preprocessed.name = "Preprocessed"
    
    #pdb.gimp_display_new (newImage)
    #return
            
    # Apply Segmentation  fx_segment_watershed 11,2,0,0,0,50,50
    pdb.plug_in_gmic_qt(newImage, Preprocessed, 1,2,
    '-v - -fx_segment_watershed 11,2,0,0,0,50,50 ')
    Painted = newImage.layers[0]
    Painted.name = "Painted thru Segmentation"
    
    #pdb.gimp_display_new (newImage)
    #return
               
    # Apply Sharpen  fx_unsharp 1,1.25,10,10,0,1,1,1,0,0,0,50,50
    pdb.plug_in_gmic_qt(newImage, Painted, 1,2,
    '-v - -fx_unsharp 1,1.25,10,10,0,1,1,1,0,0,0,50,50 ')
    Sharpened = newImage.layers[0]
    Sharpened.mode = LAYER_MODE_NORMAL   
    Sharpened.opacity = 100.0  
    Sharpened.name = "Painted (sharpened)"
   
    #pdb.gimp_display_new (newImage)
    #return
                
    # Create Base Mosaic
    # -Apply Segmentation again  fx_segment_watershed 5,2,0,0,0,50,50
    pdb.plug_in_gmic_qt(newImage, Sharpened, 1,2,
    '-v - -fx_segment_watershed 5,2,0,0,0,50,50 ')
    BaseMosaic = newImage.layers[0]
    BaseMosaic.name = "Base Mosaic"
    BaseMosaic.mode = LAYER_MODE_NORMAL   
    BaseMosaic.opacity = 100.0      
    # -use curves to remove extremes
    pdb.gimp_drawable_curves_spline (BaseMosaic, HISTOGRAM_VALUE,
                                     8, (0.0,0.1, 0.2, 0.2, 0.8,0.8, 1.0, 0.9) )
    
    #pdb.gimp_display_new (newImage)
    #return
        
    Texture1 = BaseMosaic.copy()
    newImage.add_layer (Texture1, 0)
    TextureParms='0,0.2,      115,          49,           0   ,0,0,0,1000,8,0.3'
    TextureParms="0,0.2,"+str(RED)+","+str(GRE)+","+str(BLU)+",0,0,0,1000,8,0.3"  # <<<<
    pdb.plug_in_gmic_qt(newImage, Texture1, 1,0,
    '-v - -samj_texture_coloree '+TextureParms+' ')
    Texture1 = newImage.layers[0]
    Texture1.mode = LAYER_MODE_MULTIPLY   
    Texture1.opacity = 85.0  
    Texture1.name = "samj_texture_coloree "+TextureParms
    
    #pdb.gimp_display_new (newImage) 
    #return
    
    # Duplicate 
    Texture2 = Texture1.copy()
    newImage.add_layer (Texture2, 0)
    Texture2 = newImage.layers[0]
    Texture2.mode = LAYER_MODE_MULTIPLY   
    Texture2.opacity = 85.0  
    Texture2.name = "samj_texture_coloree "+TextureParms+" (dupl)"
    
    #pdb.gimp_display_new (newImage) 
    #return

    # Preliminary outcome
    Preoutcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (Preoutcome, 0)
    Preoutcome.name = "Preliminary Outcome (sharpened)"
    pdb.plug_in_unsharp_mask (newImage, Preoutcome, 3.0, 0.75, 0)    # was  3.0, 1.5, 0) 
    
    # Apply Plasma
    PlasmaParms = "0.25,0,8,1,0,128,128,128"
    pdb.plug_in_gmic_qt(newImage, Preoutcome, 1,2,
    '-v - -fx_plasma '+PlasmaParms+' ')
    Plasma = newImage.layers[0]
    Plasma.mode = LAYER_MODE_SOFTLIGHT  #OVERLAY  
    Plasma.opacity = 50.0   #85.0   
    Plasma.name = "fx_plasma "+PlasmaParms
    
    # SourceOnTop
    SourceOnTop = Preprocessed.copy()
    newImage.add_layer (SourceOnTop, 0)
    #pdb.gimp_drawable_desaturate (SourceOnTop, DESATURATE_LUMA)
    SourceOnTop.name = "Preprocessed to overlay "
    pdb.gimp_drawable_curves_spline (SourceOnTop, HISTOGRAM_VALUE,
                                     8, (0.0,0.0, 0.25,0.1, 0.75,0.9, 1.0,1.0) )
    SourceOnTop.mode = LAYER_MODE_OVERLAY   #DISSOLVE
    SourceOnTop.opacity = 25.0  #50.0  
    LM = pdb.gimp_layer_create_mask (SourceOnTop, ADD_MASK_COPY)
    #pdb.gimp_layer_add_mask (SourceOnTop, LM)
    
    # resize image to incorporate a frame
    FrameDelta = int(WorkWH/20)
    pdb.gimp_image_resize (newImage, width+FrameDelta, height+FrameDelta,
                           FrameDelta/2, FrameDelta/2)
    
    # add a frame
    border_layer = pdb.gimp_layer_new (newImage, width+FrameDelta, height+FrameDelta,
                                       RGBA_IMAGE,"Frame ",
                                        100.0, LAYER_MODE_NORMAL)
    newImage.add_layer (border_layer, -1)
    pdb.gimp_selection_all (newImage)
    pdb.gimp_selection_shrink (newImage, FrameDelta/2)
    pdb.gimp_selection_invert (newImage)
    pdb.gimp_context_set_pattern ("Bricks")
    pdb.gimp_drawable_edit_fill (border_layer, FILL_PATTERN)
    pdb.gimp_drawable_curves_spline (border_layer, HISTOGRAM_VALUE,
                                     6, (0.0,0.0, 0.65,0.3, 1.0,1.0) )
    pdb.gimp_selection_shrink (newImage, FrameDelta/10)
    pdb.gimp_drawable_brightness_contrast (border_layer, +0.25, +0.10)
    pdb.gimp_selection_grow (newImage, FrameDelta/10)
    colour_layer = border_layer.copy()
    newImage.add_layer (colour_layer, -1)
    pdb.gimp_context_set_foreground ((RED,GRE,BLU,A))
    pdb.gimp_drawable_edit_fill (colour_layer, FILL_FOREGROUND)
    colour_layer.name = "Chosen nuance"
    colour_layer.mode = LAYER_MODE_HSL_COLOR
    colour_layer.opacity = 65.0 
    pdb.gimp_selection_none (newImage)
    
    # DO rescaling BEFORE creating the final outcome
    # user required output image size
     
    if UserOutWH==0: UserOutWH=OrigWH
    if UserOutWH != WorkWH:
        uRatio = UserOutWH*1.00/WorkWH*1.00
        uWidth = int(width*uRatio)
        uHeight = int(height*uRatio)
        pdb.gimp_image_scale (newImage, uWidth, uHeight)   # to user chosen size
     
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_IMAGE)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
        
    outcome.name = st+oname+" as a Mosaic with Texture Coloree"
    options = ["basic", "regular", "fine" ]
    optionused = " (lines look "+options[TextureLook]+")"
    outcome.name = outcome.name+optionused      
      
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleGraphics/"
copyrightYear="2022"
desc="Mosaic Texture Coloree"

register(
    "MosaicTextureColoree",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    desc+" ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
                                   "\n (0=same as input)",
                                   3000, (0, 7200, 100)),
        (PF_COLOR,  "color", "Nuance (dark)", (90, 70, 50)),     
        (PF_OPTION, "lines", "Lines look", 1, ["Basic", "Regular", "Fine" ]),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    MosaicTextureColoree,
    menu=menu
)

main()
